#
#		Python GUI - Windows - Generic
#

from Properties import overridable_property
from Containers import Container
from Applications import Application, application

class Window(Container):
    """Top-level Container."""

    _menus = []
    _document = None
    _closable = 0

    def __init__(self, style = 'standard', closable = 1, **kwds):
        Container.__init__(self, **kwds)
        self._dialog_style = style.find('dialog') >= 0
        self._closable = closable
        application()._add_window(self)

    def destroy(self):
        """Detaches the window from document and application and removes it
        from the screen."""
        self.set_document(None)
        application()._remove_window(self)
        Container.destroy(self)

    #
    #		Message handling
    #

    def next_handler(self):
        if self._document:
            return self._document
        else:
            return application()

    #
    #		Menus
    #
    
    menus = overridable_property('menus', "Menus to be available when this window is active.")

    def get_menus(self):
        return self._menus

    def set_menus(self, x):
        self._menus = x

    #
    #		Document association
    #
    
    document = overridable_property('document', "Document with which this window is associated.")

    def get_document(self):
        return self._document
    
    def set_document(self, x):
        if self._document != x:
            if self._document:
                self._document._windows.remove(self)
            self._document = x
            if self._document:
                self._document._windows.append(self)
                self.update_title()

    #
    #		Title
    #
    
    title = overridable_property('title', "Title of the window.")

    def update_title(self):
        """Update the window's title after a change in its document's title."""
        doc = self._document
        if doc:
            self.set_title(doc.title)
    
    #
    #   Positioning
    #
    
    def center(self):
        """Position the window in the centre of the screen."""
        sl, st, sr, sb = self._screen_rect()
        w, h = self.size
        l = (sr - sl - w) / 2
        t = (sb - st - h) / 2
        self.position = (l, t)
    
    centre = center

    #
    #		Menu commands
    #

    def setup_menus(self, m):
        app = application()
        if self._closable:
            if len(app._windows) > 1 or app.zero_windows_allowed():
                m.close_cmd.enabled = 1

    def close_cmd(self):
        """If this window is the only window belonging to a document,
        close the document, else destroy the window."""
        app = application()
        if len(app._windows) <= 1 and not app.zero_windows_allowed():
            #print "GWindow.close_cmd: Quitting the application" ###
            app.quit_cmd()
        else:
            doc = self._document
            if doc and len(doc._windows) <= 1:
                doc.close_cmd()
            else:
                self.destroy()

    #
    #   Other
    #
    
    def get_window(self):
        return self

    def first_dispatcher(self):
        return self
